@@ -39,13 +39,13 @@ sub _build_helper_class { 'Catalyst::Helper' }
 sub run {
     my ($self) = @_;
 
-    $self->_getopt_full_usage if !$self->ARGV->[0];
+    $self->print_usage_text if !$self->ARGV->[0];
 
     my $helper_class = $self->helper_class;
     Class::MOP::load_class($helper_class);
     my $helper = $helper_class->new( { '.newfiles' => !$self->force, mech => $self->mechanize } );
 
-    $self->_getopt_full_usage unless $helper->mk_component( $self->application_name, @{$self->extra_argv} );
+    $self->print_usage_text unless $helper->mk_component( $self->application_name, @{$self->extra_argv} );
 
 }
 
